// Header
#include "DiskStorage.h"

// Simox
#include <SimoxUtility/algorithm/string.h>

// ArmarX
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/exceptions/LocalException.h>

namespace armarx::armem::server::ltm::disk
{
    //const std::string DiskMemoryItem::Prefix = "_";
    //const std::string DiskMemoryItem::PrefixEscaped = "__";
    const std::map<std::string, std::string> DiskMemoryItem::EscapeTable = {
        {"/", "|"}
    };

    DiskMemoryItem::DiskMemoryItem(const std::filesystem::path& memoryParentPath, const std::string& memoryName, const std::filesystem::path relativePath, const MemoryEncodingMode mode) :
        memoryParentPath(memoryParentPath),
        memoryName(memoryName),
        relativeSegmentPath(relativePath),
        mode(mode)
    {
    }

    std::string DiskMemoryItem::EscapeSegmentName(const std::string& segmentName)
    {
        std::string ret = segmentName;
        //simox::alg::replace_all(ret, Prefix, PrefixEscaped);
        for (const auto& [s, r] : EscapeTable)
        {
            ret = simox::alg::replace_all(ret, s, r);
        }
        return ret;
    }

    std::string DiskMemoryItem::UnescapeSegmentName(const std::string& escapedName)
    {
        std::string ret = escapedName;
        for (const auto& [s, r] : EscapeTable) // Here we assume that noone uses the replaced char usually in the segment name... TODO
        {
            ret = simox::alg::replace_all(ret, r, s);
        }
        return ret;
    }
}
