#pragma once

#include <filesystem>
#include <map>
#include <string>

// minizip
#include <minizip/zip.h>

#include "Data.h"

namespace armarx::armem::server::ltm::disk
{
    class DiskMemoryItem
    {
    public:
        enum class MemoryEncodingMode
        {
            FILESYSTEM = 0,
            MINIZIP = 1
        };

        DiskMemoryItem() = default;
        DiskMemoryItem(const std::filesystem::path& memoryParentPath, const std::string& memoryName, const std::filesystem::path relativePath, const MemoryEncodingMode mode = MemoryEncodingMode::MINIZIP);
        virtual ~DiskMemoryItem() = default;

    protected:
        std::filesystem::path getMemoryBasePath() const
        {
            switch(mode)
            {
            case MemoryEncodingMode::FILESYSTEM:
                return memoryParentPath / memoryName;
            case MemoryEncodingMode::MINIZIP:
                return memoryParentPath / (memoryName + minizip::util::MINIZIP_SUFFIX);
            }
            return "";
        }

        static std::string EscapeSegmentName(const std::string& segmentName);
        static std::string UnescapeSegmentName(const std::string& escapedName);

    protected:
        //static const std::string Prefix;
        //static const std::string PrefixEscaped;
        static const std::map<std::string, std::string> EscapeTable;

        std::filesystem::path memoryParentPath;
        std::string memoryName;

        std::filesystem::path relativeSegmentPath;
        MemoryEncodingMode mode;
    };
} // namespace armarx::armem::server::ltm::disk
