/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "SpecializedGenerator.h"

#include <string>


namespace armarx::aron::codegenerator::cpp::generator::detail
{
    template<typename typeT, typename DerivedT>
    class AnyGenerator :
        public SpecializedGeneratorBase<typeT, DerivedT>
    {
    public:
        using SpecializedGeneratorBase<typeT, DerivedT>::SpecializedGeneratorBase;
        virtual ~AnyGenerator() = default;

        CppBlockPtr getWriteBlock(const std::string& cppAccessor, const Path& p, std::string& variantAccessor) const override
        {
            auto block_if_data = std::make_shared<CppBlock>();
            std::string escaped_accessor = this->EscapeAccessor(cppAccessor);
            variantAccessor = Generator::ARON_VARIANT_RETURN_ACCESSOR + "_" + escaped_accessor;

            block_if_data->addLine(variantAccessor + " = armarx::aron::data::readAndWrite<armarx::aron::data::FromVariantConverter<WriterT>>(" + cppAccessor + "); // of " + cppAccessor);
            return block_if_data;
        }

        CppBlockPtr getReadBlock(const std::string& cppAccessor, const std::string& variantAccessor) const override
        {
            auto block_if_data = std::make_shared<CppBlock>();
            std::string escaped_accessor = this->EscapeAccessor(cppAccessor);

            block_if_data->addLine(cppAccessor + " = armarx::aron::data::Dict::DynamicCastAndCheck(armarx::aron::data::readAndWrite<aron::data::ToVariantConverter<ReaderT>>(" + variantAccessor + ")); // of " + cppAccessor);
            return block_if_data;
        }
    };
}
