/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "Factory.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/type/variant/All.h>

namespace armarx::aron::type
{
    // Access method
    std::unique_ptr<type::Variant> VariantFactory::create(const type::dto::GenericType& aron, const Path& path) const
    {
        auto descriptor = type::Aron2Descriptor(aron);
        switch(descriptor)
        {
            case type::Descriptor::eList: return std::make_unique<type::List>(dynamic_cast<const type::dto::List&>(aron), path);
            case type::Descriptor::eDict: return std::make_unique<type::Dict>(dynamic_cast<const type::dto::Dict&>(aron), path);
            case type::Descriptor::ePair: return std::make_unique<type::Pair>(dynamic_cast<const type::dto::Pair&>(aron), path);
            case type::Descriptor::eTuple: return std::make_unique<type::Tuple>(dynamic_cast<const type::dto::Tuple&>(aron), path);
            case type::Descriptor::eObject: return std::make_unique<type::Object>(dynamic_cast<const type::dto::AronObject&>(aron), path);
            case type::Descriptor::eNDArray: return std::make_unique<type::NDArray>(dynamic_cast<const type::dto::NDArray&>(aron), path);
            case type::Descriptor::eMatrix: return std::make_unique<type::Matrix>(dynamic_cast<const type::dto::Matrix&>(aron), path);
            case type::Descriptor::eImage: return std::make_unique<type::Image>(dynamic_cast<const type::dto::Image&>(aron), path);
            case type::Descriptor::eOrientation: return std::make_unique<type::Orientation>(dynamic_cast<const type::dto::Orientation&>(aron), path);
            case type::Descriptor::ePointCloud: return std::make_unique<type::PointCloud>(dynamic_cast<const type::dto::PointCloud&>(aron), path);
            case type::Descriptor::ePose: return std::make_unique<type::Pose>(dynamic_cast<const type::dto::Pose&>(aron), path);
            case type::Descriptor::ePosition: return std::make_unique<type::Position>(dynamic_cast<const type::dto::Position&>(aron), path);
            case type::Descriptor::eQuaternion: return std::make_unique<type::Quaternion>(dynamic_cast<const type::dto::Quaternion&>(aron), path);
            case type::Descriptor::eIntEnum: return std::make_unique<type::IntEnum>(dynamic_cast<const type::dto::IntEnum&>(aron), path);
            case type::Descriptor::eInt: return std::make_unique<type::Int>(dynamic_cast<const type::dto::AronInt&>(aron), path);
            case type::Descriptor::eLong: return std::make_unique<type::Long>(dynamic_cast<const type::dto::AronLong&>(aron), path);
            case type::Descriptor::eFloat: return std::make_unique<type::Float>(dynamic_cast<const type::dto::AronFloat&>(aron), path);
            case type::Descriptor::eDouble: return std::make_unique<type::Double>(dynamic_cast<const type::dto::AronDouble&>(aron), path);
            case type::Descriptor::eString: return std::make_unique<type::String>(dynamic_cast<const type::dto::AronString&>(aron), path);
            case type::Descriptor::eBool: return std::make_unique<type::Bool>(dynamic_cast<const type::dto::AronBool&>(aron), path);
            case type::Descriptor::eTime: return std::make_unique<type::Time>(dynamic_cast<const type::dto::AronTime&>(aron), path);
            default: throw error::ValueNotValidException(__PRETTY_FUNCTION__, "Got an unknown descriptor", std::to_string((int) descriptor) + " aka " + defaultconversion::string::Descriptor2String.at(descriptor), path);
        }
    }

}
